"use strict";
const Koa = require("koa");
const jwt = require("koa-jwt");
const bodyParser = require("koa-bodyparser");
const error_middleware_1 = require("./error_middleware");
const success_middleware_1 = require("./success_middleware");
const koaCors = require("koa-cors");
const userAgent = require("koa-useragent");
const log_middleware_1 = require("./log_middleware");
class App {
    constructor(config, router) {
        this.initKoa(config, router);
    }
    initKoa(config, router) {
        const app = new Koa();
        const appPort = config.get('port');
        let jwtSecret = config.has('jwt.secret') ? config.get('jwt.secret') : null;
        app.use(bodyParser());
        if (config.has('logLevel') && (config.get('logLevel') === 'TRACE' || config.get('logLevel') === 'DEBUG')) {
            app.use(log_middleware_1.logMiddleware);
        }
        app.use(error_middleware_1.errorMiddleware);
        if (jwtSecret) {
            app.use(jwt({ secret: jwtSecret })
                .unless({
                path: [
                    new RegExp(config.get('jwt.publicPath'))
                ],
                method: 'OPTIONS'
            }));
        }
        if (!config.has('cors.enabled') || config.get('cors.enabled') !== false) {
            const corsConfig = {
                origin: '*'
            };
            if (config.has('cors.origin')) {
                corsConfig.origin = config.get('cors.origin');
            }
            if (config.has('cors.credentials')) {
                corsConfig.credentials = config.get('cors.credentials');
            }
            app.use(koaCors(corsConfig));
        }
        if (config.has('userAgent') && config.get('userAgent') === true) {
            app.use(userAgent());
        }
        app.use(router.routes());
        app.use(router.allowedMethods());
        app.use(success_middleware_1.successMiddleware);
        app.on('error', (err, ctx) => console.error('REQUEST_ERROR', err, ctx));
        process.on('uncaughtException', (err) => console.error('PROCESS_EXCEPTION', err.stack));
        if (config.has('host')) {
            const appHost = config.get('host');
            app.listen(appPort, appHost, () => console.info(`Server listening on port ${appPort} and host ${appHost}`));
        }
        else {
            app.listen(appPort, () => console.info('Server listening on port ' + appPort));
        }
        this.koa = app;
    }
}
exports.App = App;
